<?php
/**
 * WikiDB: General functions to help with the various command-line scripts.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2014, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 907 $
 */

//////////////////////// SQL HELPERS

// Helper functions for installation code.
// This was written for WikiDB, but some bits may be useful for other
// extensions.

// ApplySQLFile()
// Applies the specified SQL file to the specified DB, outputting the message
// beforehand and an indication of success afterwards.
// TODO: Do we need to provide a separate postgre version of these SQL files?
//		 Use ($wgDBtype === 'postgres') as the test, if this is required, and pick
//		 the appropriate version from within this function.
function ApplySQLFile($Message, $SQLFileName, &$DB) {
	print($Message . "...");

	$Result = $DB->sourceFile(dirname( __FILE__ ) . "/sql/" . $SQLFileName);

	if (is_string($Result)) {
		print("ERROR!\n" . $Result . "\n");
		exit(1);
	}
	else
		print("ok\n");
}

function RemoveField($Table, $Field, &$DB) {
	if (!$DB->tableExists($Table)) {
		print("Warning: Attempt to remove " . $Field
			  . " field from non-existant table, " . $Table . "!\n");
	}
	elseif (!$DB->fieldExists($Table, $Field)) {
		print("..." . $Field . " field already removed from " . $Table
			  . " table.\n");
	}
	else {
	// TODO: There is no sane way of performing a DROP COLUMN operation in SQLite
	//		 so this code will fail with an error.  Should we fail more gracefully?
	//		 Not relevant at the moment, as the only time this is used is when
	//		 upgrading from an old version of WikiDB which did not support SQLite
	//		 anyway, so will only become an issue if a column needs to be dropped
	//		 after the point that SQLite support was completed.
		print("Removing " . $Field . " field from " . $Table . " table....");
		$SQL = "ALTER TABLE " . $DB->tableName($Table)
			 . " DROP COLUMN " . $Field . ";";
		$DB->query($SQL, "WikiDB installer: RemoveField()");
		print("ok\n");
	}
}

// IsTruncateSupported()
// Returns true if the supplied Database configuration is known to support the
// TRUNCATE statement, false otherwise.
// It returns true for DB types where I know this is supported, and false for all
// other types.  This means that any system which supports TRUNCATE but which is
// not explicitly listed will not benefit from the increased performance that
// TRUNCATE can bring, though everything should nonetheless function correctly (just
// less-optimally).
function IsTruncateSupported($DB) {
	switch ($DB->getType()) {
	// DB types that are known to support the TRUNCATE statement.
		case "mysql":
			return true;

	// DB types that are known to NOT support the TRUNCATE statement.
		case "sqlite":
			return false;

	// Assume all other DB types do not support it, as this is a safer fall-back
	// (it will lead to slower code, rather than broken code as could happen if
	// we assumed the opposite).
		default:
			return false;
	}
}

// EmptyTable()
// Deletes all records from the specified $Table, using the supplied $DB connection.
// It will use the TRUNCATE statement if the current DB configuration supports it,
// as it is quicker and also resets auto-numbers.
// If not, then a standard delete() call will be used, to delete all records.
function EmptyTable($Table, $DB, $fname = "EmptyTable") {
	if (IsTruncateSupported($DB)) {
		$AffectedRows = $DB->selectField($Table, "COUNT(*)", "", $fname);
		$SQL = "TRUNCATE " . $DB->tableName($Table);
		$DB->query($SQL, $fname);
	}
	else {
		$WhereClause = "*";
		$DB->delete($Table, $WhereClause, $fname);
		$AffectedRows = $DB->affectedRows();
	}

	return $AffectedRows;
}

//////////////////////// WIKIDB HELPERS

// Based on fixLinksFromArticle()
// Reparses given article using current parser state.  You should disable
// any unnecessary tags/extensions/etc. before this is called, to improve speed.
function ReparseArticle($ArticleID) {
	global $wgTitle, $wgParser;
	$fname = "ReparseArticle";

	$wgTitle = Title::newFromID($ArticleID);
	$DB =& wfGetDB(DB_MASTER);

	if (is_null($wgTitle))
		return;

	$DB->begin();

	$Revision = Revision::newFromTitle($wgTitle);

	if ($Revision) {
	// We need this next line to stop problems with the parser referring to mTitle,
	// e.g. in areSubpagesAllowed().
		$wgParser->mTitle = $wgTitle;
		WikiDB::PageUpdated($wgTitle, $Revision->getText());
	}

	$DB->commit();
}

function ClearTableDefCache($NS = NULL) {
	$fname = "ClearTableDefCache";

	$DB =& wfGetDB(DB_WRITE);

// If no namespace is specified then we are emptying the whole table.
	if ($NS === NULL) {
		$AffectedRows = EmptyTable(pWIKIDB_tblTables, $DB, $fname);
	}
// Otherwise we are only wiping a single namespace.
	else {
		$WhereClause = array('table_namespace' => $NS);
		$DB->delete(pWIKIDB_tblTables, $WhereClause, $fname);
		$AffectedRows = $DB->affectedRows();
	}

	return $AffectedRows;
}

function ClearAllWikiDBCaches() {
	$fname = "ClearAllWikiDBCaches";

	$DB =& wfGetDB(DB_WRITE);

	$arrTables = array(pWIKIDB_tblTables, pWIKIDB_tblRows, pWIKIDB_tblFields);

	foreach ($arrTables as $Table)
		EmptyTable($Table, $DB, $fname);
}

//////////////////////// UI HELPERS

function AllowUserToCancel($Delay = 5) {
	global $options;

// If '--quick' wasn't specified, give them 5 seconds to make sure they want to
// proceed with this potentially slow process.
	if(!isset($options['quick'])) {
		print("You can abort by pressing Control-C in the next five seconds...");

		$OldVal = "";
		for ($i = $Delay; $i >= 0; $i--) {
			print_c_fixed($OldVal, $i);
			$OldVal = $i;
			sleep(1);
		}
		print("\n");
	}

}

// Replacement for print_c() in maintenance/counter.inc
// Original is buggy when number of characters in $last and $current is different,
// at least on Windows (not tested on *nix).
function print_c_fixed($last, $current) {
// On Windows backspace on its own moves the cursor, but doesn't blank out the
// value.  Therefore we do back-space (move back), space (wipe character)
// back-space (move back again) instead.
	echo str_repeat( chr(8) . " " . chr(8) , strlen( $last ) ) . $current;
}
